#version 330 core

uniform sampler2D gPosition;
uniform sampler2D gNormal;
uniform sampler2D gAlbedo;
uniform sampler2D gSSAO;

uniform vec2 screensize;

uniform vec3 V;
uniform float shininess; //TODO as gbuffer texture

uniform vec3 lightPosition;
uniform vec3 lightColor;
uniform float lightIntensity;
//uniform float specularIntensity;
uniform vec3 attenuationFactors;

uniform int useSSAO;
uniform int useShadowMap;

uniform samplerCubeShadow gShadowMap;
uniform float far_plane;

uniform vec2 near_far = vec2(0.01, 50.0);

uniform mat4 inverseView;

out vec4 fragColor;

vec3 resolveHomogene(vec4 p) { return p.xyz / p.w; }

vec2 calcTexCoords(){
	return gl_FragCoord.xy / screensize;
}

float calcAttenuation(vec3 fragmentPosition){
	float constant = attenuationFactors.x;
	float linear = attenuationFactors.y;
	float exponential = attenuationFactors.z;

	float d = length(lightPosition - fragmentPosition);

	return (constant + linear * d + exponential * d * d);
}

//via http://learnopengl.com/#!Advanced-Lighting/SSAO
float blurOcclusion(vec2 fragTexCoords){
	vec2 texelSize = 1.0 / vec2(textureSize(gSSAO, 0));
	float result = 0.0;
	for (int x = -2; x < 2; ++x) 
	{
		for (int y = -2; y < 2; ++y) 
		{
			vec2 offset = vec2(float(x), float(y)) * texelSize;
			result += texture(gSSAO, fragTexCoords + offset).r;
		}
	}
	return result / (16.0);
}

float calcShadow(vec3 fragPos)
{
	 // Get vector between fragment position and light position
	vec3 fragToLight = fragPos - resolveHomogene(inverseView * vec4(lightPosition,1));

	float depth = length(fragToLight);
	vec3 dir = fragToLight / depth; // normalize it

	depth = (depth - near_far.x) / (near_far.y - near_far.x);

	float shadow_factor = texture(gShadowMap, vec4(dir, depth));
	return shadow_factor;
}

void main()
{
	vec2 fragTexCoords = calcTexCoords();

	vec3 P = texture(gPosition, fragTexCoords).xyz;
	vec3 N = normalize(texture(gNormal, fragTexCoords).xyz);
	vec3 Cd = texture(gAlbedo, fragTexCoords).rgb;
	float occlusionFactor = blurOcclusion(fragTexCoords);

	float Id = lightIntensity;

	vec3 ambient = 0.5*Cd;
	ambient = (lightIntensity == 0.0 ? vec3(0.0) : ambient);

	
	//float Is = specularIntensity;

	

	vec3 V = normalize (V - P);
	vec3 L = normalize (lightPosition - P);
	vec3 H = normalize (V + L);
	
	float attenuationFactor = 1.0/calcAttenuation(P);

	//vec3 ambient = vec3(0.1);
	if(useSSAO == 1) Id = Id;//Id *= occlusionFactor;

	vec3 diffuse = Cd * Id * lightColor * max(0.0, dot(N, L));
	vec3 specular = vec3(0);
	if(max(0.0, dot(N, L)) > 0.0){
		specular = vec3(0);									 // Id * lightColor * pow(max(0.0, dot(N, H)), shininess) * max(0.0, dot(N, L));
	}

	float shadow = 1.0;
	if(useShadowMap == 1){
		vec3 worldP = resolveHomogene(inverseView * vec4(P,1));
		shadow = calcShadow(worldP);
	}

	vec3 color = (diffuse + specular);

	color = ambient*attenuationFactor + shadow*(diffuse)*attenuationFactor; // ambient + shadow*(diffuse + specular)*attenuationFactor;
	color *= (useSSAO == 1 ? occlusionFactor : 1.0);
	
	//vec3 fragToLight = worldP - resolveHomogene(inverseView * vec4(lightPosition,1));

	//float depth = length(fragToLight);
	//vec3 dir = fragToLight / depth; // normalize it
	
	fragColor = vec4(color, 1.0);


	//fragColor = vec4(N,1.0);// * inverseView;//inverse(inverseView);
}